@@ -186,11 +186,14 @@ class ServiceStartType:
         self.set_start_type(win32service.SERVICE_SYSTEM_START, wait_for_seconds)
 
     def set_start_type(
-        self, start_type: Literal[2, 0, 3, 4, 1], wait_for_seconds: int = 10
+        self,
+        start_type: Literal[2, 0, 3, 4, 1],
+        wait_for_seconds: int = 10,
     ):
         win32serviceutil.ChangeServiceConfig(
             None, self.service_name, startType=start_type
         )
+
         wait_for_service_config_status(self.service_name, start_type, wait_for_seconds)
 
 
@@ -199,6 +202,16 @@ class ServiceState:
     value: Literal[5, 6, 7, 4, 2, 3, 1]
     service_name: str
 
+    value_to_ms_name = {
+        win32service.SERVICE_CONTINUE_PENDING: "SERVICE_CONTINUE_PENDING",
+        win32service.SERVICE_PAUSE_PENDING: "SERVICE_PAUSE_PENDING",
+        win32service.SERVICE_PAUSED: "SERVICE_PAUSED",
+        win32service.SERVICE_RUNNING: "SERVICE_RUNNING",
+        win32service.SERVICE_START_PENDING: "SERVICE_START_PENDING",
+        win32service.SERVICE_STOP_PENDING: "SERVICE_STOP_PENDING",
+        win32service.SERVICE_STOPPED: "SERVICE_STOPPED",
+    }
+
     @property
     def continue_pending(self):
         return self.value == win32service.SERVICE_CONTINUE_PENDING
@@ -227,6 +240,10 @@ class ServiceState:
     def stopped(self):
         return self.value == win32service.SERVICE_STOPPED
 
+    @property
+    def type(self):
+        return self.value_to_ms_name.get(self.value, "UNKNOWN")
+
     def start(self, wait_for_seconds: int = 10):
         win32serviceutil.StartService(self.service_name)
         win32serviceutil.WaitForServiceStatus(
@@ -239,6 +256,9 @@ class ServiceState:
             self.service_name, win32service.SERVICE_STOPPED, wait_for_seconds
         )
 
+    def __repr__(self):
+        return f"<ServiceState: {self.value} ({self.type})>"
+
 
 @dataclass(frozen=True)
 class ServiceControls:
@@ -385,8 +405,10 @@ class ServiceControlManager:
 
     def __post_init__(self):
         # early raise if we don't have correct permissions
-        with self.get_handle() as _:
+        logging.debug("Ensuring we have correct permissions for %s", self.service_name)
+        with _get_svc_handle(self.service_name) as _:
             pass
+        logging.debug("We have correct permissions for %s", self.service_name)
 
     def start(self) -> None:
         win32serviceutil.StartService(self.service_name)
@@ -397,59 +419,14 @@ class ServiceControlManager:
 
     @property
     def status(self) -> ServiceStatus:
-        with self.get_handle() as handle:
+        with _get_svc_handle(self.service_name) as handle:
             return ServiceStatus.create(handle, self.service_name)
 
     @property
     def config(self) -> ServiceConfig:
-        with self.get_handle() as handle:
+        with _get_svc_handle(self.service_name) as handle:
             return ServiceConfig.create(handle, self.service_name)
 
-    @contextmanager
-    def get_handle(self):
-        # some various combinations of permissions that I've found work to get a handle
-        permission_combos = (
-            (win32service.SC_MANAGER_ALL_ACCESS, win32service.SERVICE_ALL_ACCESS),
-            (win32service.SC_MANAGER_CONNECT, win32service.SERVICE_CHANGE_CONFIG),
-            (win32con.GENERIC_READ, win32service.SERVICE_ALL_ACCESS),
-        )
-        hs = None
-        svc_ctrl_manager = None
-
-        try:
-            for idx, values in enumerate(permission_combos):
-                sc_perm, svc_perm = values
-                logging.debug(
-                    "attempting to get service handle for %s", self.service_name
-                )
-                svc_ctrl_manager = win32service.OpenSCManager(None, None, sc_perm)
-                logging.debug("opening service")
-
-                try:
-                    hs = win32service.OpenService(
-                        svc_ctrl_manager, self.service_name, svc_perm
-                    )
-
-                except pywintypes.error as e:
-                    if idx == len(permission_combos) - 1:
-                        raise
-                    if e.winerror == winerror.ERROR_ACCESS_DENIED:
-                        logging.debug("access denied, trying another permission combo")
-                        continue
-
-                if hs:
-                    logging.debug("got service handle for %s", self.service_name)
-
-                    yield hs
-                    break
-        finally:
-            if hs:
-                logging.debug("closing service")
-                win32service.CloseServiceHandle(hs)
-            if svc_ctrl_manager:
-                logging.debug("closing service manager")
-                win32service.CloseServiceHandle(svc_ctrl_manager)
-
 
 @contextmanager
 def stopped_service(
@@ -470,120 +447,59 @@ def stopped_service(
     service = ServiceControlManager(service_name)
 
     # stop the service
-    previous_state = service.status.state.value
+    previous_state = service.status.state
+    logging.debug("Current service state: %s", previous_state)
 
     try:
+        logging.debug("Stopping service %s", service_name)
         service.status.state.stop(wait_for_seconds=wait_for)
     except pywintypes.error as e:
         if e.winerror == winerror.ERROR_SERVICE_NOT_ACTIVE and ignore_stopped:
             # service is already stopped and we don't care
+            logging.debug("Service %s is already stopped, ignoring.", service_name)
             pass
         else:
             # service is already stopped or some other error
             raise
 
     # disable the service
-    previous_start_type = service.config.start_type.value
+    previous_start_type = service.config.start_type
+    logging.debug("Current service start type: %s", previous_start_type)
 
     if also_disable:
+        logging.debug("Disabling service %s", service_name)
         service.config.start_type.set_disabled(wait_for_seconds=wait_for)
 
     # yield the service
     yield service
 
     # enable the service
-    if also_disable:
+    if also_disable and not previous_start_type.disabled:
+        logging.debug(
+            "Restoring start type to %s for service %s",
+            previous_start_type,
+            service_name,
+        )
         service.config.start_type.set_start_type(
-            previous_start_type, wait_for_seconds=wait_for
+            previous_start_type.value, wait_for_seconds=wait_for
         )
 
     # start the service
-    if previous_state in (
-        win32service.SERVICE_RUNNING,
-        win32service.SERVICE_START_PENDING,
-        win32service.SERVICE_CONTINUE_PENDING,
+    if any(
+        [
+            previous_state.running,
+            previous_state.start_pending,
+            previous_state.continue_pending,
+        ]
     ):
+        logging.debug("Restarting service %s", service_name)
         service.status.state.start(wait_for_seconds=wait_for)
-
-
-def is_admin():
-    # noinspection PyBroadException
-    try:
-        return ctypes.windll.shell32.IsUserAnAdmin()
-    except Exception:
-        return False
-
-
-def request_admin_privileges(exec_with: Optional[List[str]] = None) -> None:
-    """
-    Requests admin privileges by showing a UAC prompt.
-
-    :return: None
-    """
-    if not is_admin():
-        # if not provided with a command line, join the current command line
-        # with the current arguments
-        if not exec_with:
-            exec_with = sys.argv
-        logging.debug("requesting admin privileges")
-        logging.debug("running with executable: %s", sys.executable)
-        logging.debug("running with arguments: %s", exec_with)
-        if sys.platform == "win32":
-            # using subprocess, re-run with "runas" to get admin privileges
-            subprocess.Popen(["runas", sys.executable, *exec_with], shell=True)
-            # win32api.ShellExecute(
-            #         0, "runas", sys.executable, exec_with, None, win32con.SW_SHOW
-            # )
-            # ctypes.windll.shell32.ShellExecuteW(
-            #         None, "runas", sys.executable, exec_with, None, 1
-            # )
-            sys.exit(0)
-
-
-def is_running(service_name: str) -> bool:
-    """
-    Checks if a service is running.
-
-    :param service_name: Name of the service to check.
-    :return: True if the service is running, otherwise False.
-    """
-    return (
-        win32serviceutil.QueryServiceStatus(service_name)[1]
-        == win32service.SERVICE_RUNNING
-    )
-
-
-def is_disabled(service_name: str) -> bool:
-    """
-    Checks if a service is disabled.
-
-    :param service_name: Name of the service to check.
-    :return: True if the service is disabled, otherwise False.
-    """
-    return (
-        win32service.QueryServiceConfig(service_name)[1]
-        == win32service.SERVICE_DISABLED
-    )
-
-
-def start_service(service_name: str) -> None:
-    """
-    Starts a service.
-
-    :param service_name: Name of the service to start.
-    :return: None
-    """
-    win32serviceutil.StartService(service_name)
-
-
-def stop_service(service_name: str) -> None:
-    """
-    Stops a service.
-
-    :param service_name: Name of the service to stop.
-    :return: None
-    """
-    win32serviceutil.StopService(service_name)
+    else:
+        logging.debug(
+            "Service %s previously had a state of %s, not starting.",
+            service_name,
+            previous_state,
+        )
 
 
 def wait_for_service_config_status(
@@ -599,48 +515,57 @@ def wait_for_service_config_status(
     :param status: Status to wait for.
     :return: None
     """
-    with get_service_handle(service_name) as service_handle:
-        for i in range(wait_seconds * 4):
-            now_status = win32service.QueryServiceConfig(service_handle)[1]
-            if now_status == status:
-                break
-            win32api.Sleep(250)
-        else:
-            logging.error(
-                "service %s did not reach status %s. It remained %s after %s seconds.",
-                service_name,
-                status,
-                now_status,
-                wait_seconds,
-            )
-            raise pywintypes.error(
-                winerror.ERROR_SERVICE_REQUEST_TIMEOUT,
-                "QueryServiceConfig",
-                win32api.FormatMessage(winerror.ERROR_SERVICE_REQUEST_TIMEOUT)[:-2],
-            )
+    for i in range(wait_seconds * 4):
+        with _get_svc_handle(service_name) as handle:
+            now_status = win32service.QueryServiceConfig(handle)[1]
+        if now_status == status:
+            break
+        win32api.Sleep(250)
+    else:
+        raise pywintypes.error(
+            winerror.ERROR_SERVICE_REQUEST_TIMEOUT,
+            "QueryServiceConfig",
+            win32api.FormatMessage(winerror.ERROR_SERVICE_REQUEST_TIMEOUT)[:-2],
+        )
 
 
 @contextmanager
-def get_service_handle(service_name: str) -> None:
-    """
-    Gets a service handle.
-
-    :param service_name: Name of the service to get the handle for.
-    :return: None
-    """
-    logging.debug("getting service handle")
-    hscm = win32service.OpenSCManager(None, None, win32service.SC_MANAGER_ALL_ACCESS)
+def _get_svc_handle(service_name: str) -> None:
+    # some various combinations of permissions that I've found work to get a handle
+    permission_combos = (
+        (win32service.SC_MANAGER_ALL_ACCESS, win32service.SERVICE_ALL_ACCESS),
+        (win32service.SC_MANAGER_CONNECT, win32service.SERVICE_CHANGE_CONFIG),
+        (win32con.GENERIC_READ, win32service.SERVICE_ALL_ACCESS),
+    )
     hs = None
-    logging.debug("opening service")
+    svc_ctrl_manager = None
+
     try:
-        hs = win32service.OpenService(
-            hscm, service_name, win32service.SERVICE_ALL_ACCESS
-        )
-        logging.debug("yielding service handle")
-        yield hs
+        for idx, values in enumerate(permission_combos):
+            sc_perm, svc_perm = values
+            logging.debug("attempting to get service handle for %s", service_name)
+            svc_ctrl_manager = win32service.OpenSCManager(None, None, sc_perm)
+            logging.debug("opening service %s", service_name)
+
+            try:
+                hs = win32service.OpenService(svc_ctrl_manager, service_name, svc_perm)
+
+            except pywintypes.error as e:
+                if idx == len(permission_combos) - 1:
+                    raise
+                if e.winerror == winerror.ERROR_ACCESS_DENIED:
+                    logging.debug("access denied, trying another permission combo")
+                    continue
+
+            if hs:
+                logging.debug("got service handle for %s", service_name)
+
+                yield hs
+                break
     finally:
         if hs:
-            logging.debug("closing service")
+            logging.debug("closing service %s", service_name)
             win32service.CloseServiceHandle(hs)
-        logging.debug("closing service manager")
-        win32service.CloseServiceHandle(hscm)
+        if svc_ctrl_manager:
+            logging.debug("closing service manager")
+            win32service.CloseServiceHandle(svc_ctrl_manager)
